
class DataStoreCluster():

    def __init__(self, spartacloud_api_instance, target_node:int=None):
        self.spartacloud_api_instance = spartacloud_api_instance
        self.target_node = target_node
        
    def status(self) -> dict:
        '''
        Get redis data store cluster status
        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        return self.spartacloud_api_instance.query_service('get_data_store_cluster_status', data_dict)

    def keys(self) -> dict:
        '''
        Get redis data store cluster keys
        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        return self.spartacloud_api_instance.query_service('get_data_store_cluster_keys', data_dict)

    # *****************************************************************************
    # OPERATION ENDPOINT
    # *****************************************************************************
    def get(self, key) -> dict:
        '''
        Get redis data store cluster get value (can receive multiple value from different nodes)
        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['key'] = key
        data_dict['operation'] = 'get'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)
    
    def set(self, key, value) -> dict:
        '''
        Put data to redis (if target_node not defined, put to all nodes)
        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['key'] = key
        data_dict['value'] = value
        data_dict['operation'] = 'set'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)
    
    def delete(self, *keys) -> dict:
        '''
        Delete data to redis
        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['keys'] = keys
        data_dict['operation'] = 'delete'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)
    
    def rpush(self, key, *values) -> dict:
        '''

        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['key'] = key
        data_dict['values'] = values
        data_dict['operation'] = 'rpush'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)
    
    def lrange(self, key, start, end) -> dict:
        '''
        
        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['key'] = key
        data_dict['start'] = start
        data_dict['end'] = end
        data_dict['operation'] = 'lrange'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)
    
    def hset(self, key, field, value) -> dict:
        '''
        
        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['key'] = key
        data_dict['field'] = field
        data_dict['value'] = value
        data_dict['operation'] = 'hset'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)
    
    def hget(self, key, field) -> dict:
        '''
        
        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['key'] = key
        data_dict['field'] = field
        data_dict['operation'] = 'hget'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)
    
    def sadd(self, key, *members) -> dict:
        '''
       
        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['key'] = key
        data_dict['members'] = members
        data_dict['operation'] = 'sadd'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)

    def smembers(self, key) -> dict:
        '''

        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['key'] = key
        data_dict['operation'] = 'smembers'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)

    def zadd(self, key, mapping) -> dict:
        '''

        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['key'] = key
        data_dict['mapping'] = mapping
        data_dict['operation'] = 'zadd'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)

    def zrangebyscore(self, key, min, max) -> dict:
        '''
 
        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['key'] = key
        data_dict['min'] = min
        data_dict['max'] = max
        data_dict['operation'] = 'zrangebyscore'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)

    def expire(self, key, seconds) -> dict:
        '''

        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['key'] = key
        data_dict['seconds'] = seconds
        data_dict['operation'] = 'expire'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)

    def ttl(self, key) -> dict:
        '''

        '''
        data_dict = self.spartacloud_api_instance.get_common_api_params()
        data_dict['target_node'] = self.target_node
        data_dict['key'] = key
        data_dict['operation'] = 'ttl'
        return self.spartacloud_api_instance.query_service('operation_data_store_cluster', data_dict)



import os
import base64
import json
import hashlib
import requests
from cryptography.fernet import Fernet

def get_ws_settings(api_key:str) -> list:
    '''
    Decrypt auth cmd
    '''
    f = Fernet(get_keygen_fernet().encode('utf-8'))
    encrypted_data = base64.b64decode(api_key)
    decrypted_data = f.decrypt(encrypted_data).decode('utf-8')
    data_list = decrypted_data.split('-')
    domain_or_ip = data_list[1]
    http_port = data_list[2]
    api_token_id = data_list[3]
    redis_port = data_list[4]
    redis_pwd = data_list[5]
    return [domain_or_ip, http_port, api_token_id, redis_port, redis_pwd]

def get_keygen_fernet() -> str:
    '''
    Get encryption key to ask to the master node token
    '''
    keygen_fernet = 'spartacloud-api-key'
    key = keygen_fernet.encode('utf-8')
    key = hashlib.md5(key).hexdigest()
    key = base64.b64encode(key.encode('utf-8'))
    return key.decode('utf-8')

def request_service(spartacloud_api_intance, service_name:str, data_dict:dict) -> dict:
    '''
    Web service request
    '''
    data_dict['api_service'] = service_name
    json_data_params = {
        'jsonData': json.dumps(data_dict)
    }
    headers = {
        "Content-Type": "application/json"
    }
    url = f"{spartacloud_api_intance.domain_or_ip}:{spartacloud_api_intance.http_port}/api_web_service"
    # print("url  > "+str(url))
    res_req = requests.post(url, json=json_data_params, headers=headers)
    status_code = res_req.status_code
    if status_code != 200:
        print(f"An error occurred, status_code: {status_code}")
        return {
            'res': -1,
            'status_code': status_code,
        }

    return json.loads(res_req.text)

def upload_resources(spartacloud_api_intance, data_dict:dict, file_path:str) -> dict:
    '''
    Upload resources (file or folder)
    '''

    def upload_func(files):
        json_data_params = {
            'jsonData': json.dumps(data_dict)
        }
        headers = {
            "Content-Type": "application/json"
        }
        url = f"{spartacloud_api_intance.domain_or_ip}:{spartacloud_api_intance.http_port}/api_web_service"
        # print("url  > "+str(url))
        res_req = requests.post(url, json=json_data_params, headers=headers, files=files)
        status_code = res_req.status_code
        if status_code != 200:
            print(f"An error occurred, status_code: {status_code}")
            return {
                'res': -1,
                'status_code': status_code,
            }

        return json.loads(res_req.text)
    
    data_dict['api_service'] = 'upload'
    is_file = False
    if os.path.isfile(file_path):
        is_file = True

    if is_file:
        file_name, file_extension = os.path.splitext(os.path.basename(file_path))
        files = {'file': (f"{file_name}.{file_extension}", open(file_path, 'rb'))}
        return upload_func(files)
    else: # We are dealing with a folder, we are going to recursively upload each file
        pass
    
    


import redis

class DataStore:

    def __init__(self, host='localhost', port=6381, password=None, db=0):
        '''
        
        '''
        self.host = host
        self.port = port
        self.db = db
        self.r = redis.StrictRedis(host=host, port=port, password=password, db=db)

    def get_keys(self) -> list:
        '''
        
        '''
        keys = self.r.keys('*')
        return [key.decode() for key in keys]
    
    def get_value(self, key):
        '''
        
        '''
        return self.r.get(key).decode()


import os
from spartacloud.spartacloud_api_utils import get_ws_settings, request_service
from spartacloud.datastore_api import DataStore as DataStore
from spartacloud.datastore_api_cluster import DataStoreCluster as DataStoreCluster

class Spartacloud:

    def __init__(self, api_key=None):
        self._set_api_key(api_key) # This key can be found on the network panel of Spartacloud GUI 

    def _set_api_key(self, api_key=None):
        '''
        Set api key
        '''
        api_key_env = os.getenv('SPARTACLOUD_API_KEY', None)
        if api_key_env is not None:
            self.api_key = api_key_env
            self._prepare_ws_settings()
            return
        
        if api_key is not None:
            self.api_key = api_key
            self._prepare_ws_settings()
            return
        
    def _prepare_ws_settings(self):
        '''
        Prepare web services settings
        '''
        if hasattr(self, 'api_key'):
            self.domain_or_ip, self.http_port, self.api_token_id, _, _ = get_ws_settings(self.api_key)
        
    def override_settings(self, domain_or_ip=None, http_port=None, api_token_id=None):
        '''
        Override api settings
        '''
        if domain_or_ip is None or http_port is None or api_token_id is None:
            raise Exception('Invalid parameters. domain_or_ip, http_port and api_token_id parameters must be valid')
        self.domain_or_ip = domain_or_ip
        self.http_port = http_port
        self.api_token_id = api_token_id

    def get_common_api_params(self) -> dict:
        return {
            'api_token_id': self.api_token_id,
        }
    
    def get_computer_nodes(self):
        '''
        Get list of cluster nodes
        '''
        data_dict = self.get_common_api_params()
        return self.query_service('get_computer_nodes', data_dict)

    def get_metrics_cluster(self):
        '''
        Get cluster metrics of the cluster (all nodes)
        '''
        data_dict = self.get_common_api_params()
        return self.query_service('get_metrics_cluster', data_dict)
    
    def get_status_cluster(self):
        '''
        Get status of the cluster (all nodes)
        '''
        data_dict = self.get_common_api_params()
        return self.query_service('get_status_cluster', data_dict)

    def get_cpu_cluster(self):
        '''
        Get cluster cpu metrics of the cluster (all nodes)
        '''
        data_dict = self.get_common_api_params()
        return self.query_service('get_cpu_cluster', data_dict)

    def get_mem_cluster(self):
        '''
        Get cluster cpu ram of the cluster (all nodes)
        '''
        data_dict = self.get_common_api_params()
        return self.query_service('get_mem_cluster', data_dict)
    
    def get_disk_cluster(self, path_disk='/host'):
        '''
        Get cluster disk of the cluster (all nodes)
        '''
        data_dict = self.get_common_api_params()
        data_dict['path_disk'] = path_disk
        return self.query_service('get_disk_cluster', data_dict)
    
    def distribute_job(self, job_cmd, job_name=None, target_node=None, target_cpu=None, target_mem=None, store_stdout=True, b_rerun=False):
        '''
        Distribute jobs to the spartacloud cluster (cluster entrypoint is worker node)
        '''
        if job_cmd is None:
            raise Exception('You must add a command. It can be a python script, a docker exec command, a shell command etc...')

        data_dict = self.get_common_api_params()
        data_dict['job_cmd'] = job_cmd
        data_dict['job_name'] = job_name
        data_dict['target_node'] = target_node
        data_dict['target_cpu'] = target_cpu
        data_dict['target_mem'] = target_mem
        data_dict['store_stdout'] = store_stdout
        data_dict['b_rerun'] = b_rerun
        return self.query_service('distribute_job', data_dict)
    
    def get_jobs(self):
        '''
        Get list of my jobs
        '''
        data_dict = self.get_common_api_params()
        return self.query_service('get_jobs', data_dict)

    def get_jobs_status(self):
        '''
        Get status of all jobs
        '''
        data_dict = self.get_common_api_params()
        return self.query_service('get_jobs_status', data_dict)
    
    def cancel_job(self, uuid):
        '''
        Cancel a specific job
        '''
        data_dict = self.get_common_api_params()
        if uuid is not None:
            data_dict['uuid'] = uuid
            return self.query_service('cancel_job', data_dict)
        
        err_msg = 'uuid is None. You must specify the uuid as input'
        print(err_msg)
        return {
            'res': -1,
            'errorMsg': err_msg
        }
    
    def cancel_all_jobs(self):
        '''
        Cancel all jobs
        '''
        data_dict = self.get_common_api_params()
        return self.query_service('cancel_all_jobs', data_dict)
    
    def get_job_std(self, uuid):
        '''
        Get job stdout and stderr
        '''
        data_dict = self.get_common_api_params()
        if uuid is not None:
            data_dict['uuid'] = uuid
            return self.query_service('get_job_std', data_dict)

        err_msg = 'uuid is None. You must specify the uuid as input'
        print(err_msg)
        return {
            'res': -1,
            'errorMsg': err_msg
        }

    def delete_job(self, uuid):
        '''
        Delete a job
        '''
        data_dict = self.get_common_api_params()
        if uuid is not None:
            data_dict['uuid'] = uuid
            return self.query_service('delete_job', data_dict)

        err_msg = 'uuid is None. You must specify the uuid as input'
        print(err_msg)
        return {
            'res': -1,
            'errorMsg': err_msg
        }
    
    def put_resource(self, path_to_upload, path_destination):
        '''
        TODO SPARTACLOUD: To implement
        path_destination: relative to /spartacloud_shared/ directory 
        '''
        pass
    
    def get_resource(self, resource_path):        
        '''
        TODO SPARTACLOUD: To implement
        resource_path: relative to /spartacloud_shared/ directory 
        '''
        pass

    def get_data_store(self, host=None, port=None, password=None, db=0) -> DataStore:
        '''
        Get redis data store object
        '''
        if host is None:
            _, _, _, port, password = get_ws_settings(self.api_key)

        data_store = DataStore(host, port, password, db=db)
        return data_store
    
    def get_data_store_cluster(self, target_node:int=None):
        '''
        
        '''
        return DataStoreCluster(spartacloud_api_instance=self, target_node=target_node)
        
    def query_service(self, service_name:str, data_dict:dict) -> dict:
        '''
        POST requests
        '''
        return request_service(self, service_name=service_name, data_dict=data_dict)
    
    def help(self) -> list:
        '''
        
        '''
        methods = [method for method in dir(self) if callable(getattr(self, method))]
        return methods